/*-
 * Copyright (c) 2008  Marin Atanasov  <dnaeon@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <unistd.h>
# include <getopt.h>
# include <sys/param.h>
# include "fnode.h"
# include "explode.h"
# include "pkg_add_it.h"
# include "scan_directory.h"
# include "scan_index_file.h"

int main (int argc, char *argv[])
{
   char *cwd;
   
   /* used to store information about the command line arguments */
   bool_t local, foundopt, listonly; /* local - local or remote install? */
                                     /* foundopt - option found or not? */
                                     /* listonly - install or list mode? */
   int chopt, numfiles, index;
   fnode_t *head, *pkgfile;
   
   /* options struct */
   static struct option longopts[] = {
      { "version", no_argument, NULL, 'v' },
      { "help", no_argument, NULL, 'h' },
      { "list", no_argument, NULL, 'L' },
      { "local", required_argument, NULL, 'l' },
      { "remote", required_argument, NULL, 'r' },
      { NULL, 0, NULL, 0 }
   } ;
        
   index = numfiles = 0;  
   local = foundopt = listonly = FALSE;
   
   /* save the current dir */
   if (!(cwd = getcwd(NULL, MAXPATHLEN + 1))) {
      perror ("getcwd");
      exit (1);
   }
   
   /* check for valid command line arguments */
   while (!foundopt && ((chopt = getopt_long(argc, argv, CL_ARGS, longopts, NULL)) != -1)) {
      switch (chopt) {
         case 'h':
            usage();
            exit (0);
            break;   /* never reached - added only to follow standarts */
         case 'v':
         # ifdef __STDC__
            fprintf (stdout, "==>> %s v%s : built on %s at %s\n", PROGNAME, PROGVERS, __DATE__, __TIME__);
         # endif
            exit (0);
            break;   /* never reached - added only to follow standarts */
         case 'L':
            listonly = TRUE; /* list mode only - no install */
            break;
         case 'l':   
            /* make some checks first */
            if (argv[optind] == NULL) {
               fprintf (stderr, "==>> You have to specify [pkg-name]\n");
               fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
               exit (1);
            }
            if (argc > optind+1) {
               fprintf (stderr, "==>> Too many arguments passed\n");
               fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
               exit (1);
            }
            if (!dir_exists(optarg)) {
               fprintf (stderr, "==>> '%s' is not a directory\n", optarg);
               fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
               exit (1);
            }
            
            /* perform a local pattern search */
            local = foundopt = TRUE;
                        
            fnode_push (&head, argv[optind], optarg, NULL, NULL);
            head->next = NULL;
            numfiles = scan_directory (optarg, argv[optind], &head);
            break;
         case 'r':
            /* make some checks first */
            if (argv[optind - 1] == NULL) {
               fprintf (stderr, "==>> You have to specify [pkg-name]\n");
               fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
               exit (1);
            }
            if (argc > optind) {
               fprintf (stderr, "==>> Too many arguments passed\n");
               fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
               exit (1);
            }
         
            /* perform a remote pattern search */
            foundopt = TRUE;
            
            fnode_push (&head, optarg, ".", NULL, NULL);
            head->next = NULL;
            numfiles = scan_index_file (optarg, &head);
            break;
         default:
            exit (1);   /* no valid options were found */
      }
   }     
   
   /* check if there were any command line options at all */
   if ((optind == 1) || (listonly && optind == 2)) {
      fprintf (stderr, "==>> Invalid call to %s\n", PROGNAME);
      fprintf (stderr, "==>> Run %s --help for a list of available options\n", PROGNAME);
      exit (1);
   }        
   
   if (!numfiles) {
      fprintf (stderr, "==>> No matching packages were found..\n");
      fprintf (stderr, "==>> Perhaps a wrong package name given?\n");
   }
   
   else {
      /* display the list of packages */
      fnode_display (head, numfiles);
      
      /* install a package? */
      if (!listonly) {
         index = fnode_choose_pkg (numfiles);
         fnode_locate (head, index, &pkgfile);
         install_cmd (pkgfile, local);
      }
   }
   
   /* return to where we were */
   chdir (cwd);
   
   /* deallocate the stack */
   fnode_shutdown (head);
   
   return 0;
}

/* prints the usage of the program */
void usage (USAGEARGS)
{
   fprintf (stderr, "Usage: %s [options=vhL] [mode=l|r] [pkg-dir] [pkg-name]\n\n", PROGNAME);
   fprintf (stderr, "%s - %s, v%s\n", PROGNAME, PROGDESC, PROGVERS);
# ifdef __STDC__
   printf ("%s v%s : built on %s at %s\n", PROGNAME, PROGVERS, __DATE__, __TIME__);
# endif
   fprintf (stderr, "\nAvailable options:\n");
   fprintf (stderr, "--list|-L      Use list mode only - no install\n");
   fprintf (stderr, "--version|-v   Prints the current version of the program\n");
   fprintf (stderr, "--help|-h      Displays this help info\n\n");
   fprintf (stderr, "[mode]         Specifies the working mode, either --local or --remote\n");
   fprintf (stderr, "--local|-l     Specifies a local pattern search\n");
   fprintf (stderr, "--remote|-r    Specifies a remote pattern search\n\n");
   fprintf (stderr, "[pkg-dir]      Specifies the directory containing package files (.tbz)\n");
   fprintf (stderr, "               Used only with the `--local` option\n");
   fprintf (stderr, "[pkg-name]     Specifies the package name\n");
   fprintf (stderr, "\nFor example usage and more information read %s(1)\n", PROGNAME);
}

/* installs the chose package */
void install_cmd (INSTALLCMDARGS)
{
   char *category, *ftppath, buf[MAXPATHLEN + 1];
   explode_t *expl_head, *expl_tail;
   int expl_count = 0;
   
   /* local installation */
   if (local)
      snprintf (buf, sizeof(buf), "%s %s/%s", CMDLOCAL, pkgfile->fpath, pkgfile->fname);
   /* remote installation */
   else {
      explode_push_queue (pkgfile->fpath, &expl_head);
      expl_tail = expl_head;
      expl_count = explode (' ', pkgfile->fpath, &expl_tail);
      
      ftppath = build_ftp_path ();
      category = (expl_count == 0) ? expl_head->str : expl_head->next->str;
      
      explode_destroy_queue (expl_head);
      
      snprintf (buf, sizeof(buf), "%s %s%s/%s.tbz", CMDREMOTE, ftppath, category, pkgfile->fname);
   }
     
   system (buf);
}
